#colours {
  color1: #fea;
  color2: #ffeeaa;
  color3: rgba(255, 238, 170, 0.1);
  @color1: #fea;
  string: "@{color1}";
}
dimensions {
  val: 0.1px;
  val: 0em;
  val: 4cm;
  val: 0.2;
  val: 5;
  angles-must-have-unit: 0deg;
  width: auto\9;
}